package sun.security.provider.certpath;

import java.util.Collections;
import java.util.Iterator;
import java.util.Set;
import java.security.cert.PolicyQualifierInfo;
import java.util.HashSet;
import java.security.cert.PolicyNode;

final class PolicyNodeImpl implements PolicyNode
{
    private static final String ANY_POLICY = "2.5.29.32.0";
    private PolicyNodeImpl mParent;
    private HashSet<PolicyNodeImpl> mChildren;
    private String mValidPolicy;
    private HashSet<PolicyQualifierInfo> mQualifierSet;
    private boolean mCriticalityIndicator;
    private HashSet<String> mExpectedPolicySet;
    private boolean mOriginalExpectedPolicySet;
    private int mDepth;
    private boolean isImmutable;
    
    PolicyNodeImpl(PolicyNodeImpl mParent, String mValidPolicy, Set<PolicyQualifierInfo> c, boolean mCriticalityIndicator, Set<String> c2, boolean b) {
        this.isImmutable = false;
        this.mParent = mParent;
        this.mChildren = new HashSet();
        if (mValidPolicy != null) {
            this.mValidPolicy = mValidPolicy;
        }
        else {
            this.mValidPolicy = "";
        }
        if (c != null) {
            this.mQualifierSet = new HashSet(c);
        }
        else {
            this.mQualifierSet = new HashSet();
        }
        this.mCriticalityIndicator = mCriticalityIndicator;
        if (c2 != null) {
            this.mExpectedPolicySet = new HashSet(c2);
        }
        else {
            this.mExpectedPolicySet = new HashSet();
        }
        this.mOriginalExpectedPolicySet = !b;
        if (this.mParent != null) {
            this.mDepth = this.mParent.getDepth() + 1;
            this.mParent.addChild(this);
        }
        else {
            this.mDepth = 0;
        }
    }
    
    PolicyNodeImpl(PolicyNodeImpl policyNodeImpl, PolicyNodeImpl policyNodeImpl2) {
        this(policyNodeImpl, policyNodeImpl2.mValidPolicy, policyNodeImpl2.mQualifierSet, policyNodeImpl2.mCriticalityIndicator, policyNodeImpl2.mExpectedPolicySet, false);
    }
    
    @Override
    public PolicyNode getParent() {
        return this.mParent;
    }
    
    @Override
    public Iterator<PolicyNodeImpl> getChildren() {
        return Collections.unmodifiableSet(this.mChildren).iterator();
    }
    
    @Override
    public int getDepth() {
        return this.mDepth;
    }
    
    @Override
    public String getValidPolicy() {
        return this.mValidPolicy;
    }
    
    @Override
    public Set<PolicyQualifierInfo> getPolicyQualifiers() {
        return Collections.unmodifiableSet(this.mQualifierSet);
    }
    
    @Override
    public Set<String> getExpectedPolicies() {
        return Collections.unmodifiableSet(this.mExpectedPolicySet);
    }
    
    @Override
    public boolean isCritical() {
        return this.mCriticalityIndicator;
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(this.asString());
        Iterator<PolicyNodeImpl> iterator = this.mChildren.iterator();
        while (iterator.hasNext()) {
            sb.append((Object)iterator.next());
        }
        return sb.toString();
    }
    
    boolean isImmutable() {
        return this.isImmutable;
    }
    
    void setImmutable() {
        if (this.isImmutable) {
            return;
        }
        Iterator<PolicyNodeImpl> iterator = this.mChildren.iterator();
        while (iterator.hasNext()) {
            ((PolicyNodeImpl)iterator.next()).setImmutable();
        }
        this.isImmutable = true;
        return;
    }
    
    private void addChild(PolicyNodeImpl e) {
        if (this.isImmutable) {
            throw new IllegalStateException("PolicyNode is immutable");
        }
        this.mChildren.add(e);
        return;
    }
    
    void addExpectedPolicy(String e) {
        if (this.isImmutable) {
            throw new IllegalStateException("PolicyNode is immutable");
        }
        if (this.mOriginalExpectedPolicySet) {
            this.mExpectedPolicySet.clear();
            this.mOriginalExpectedPolicySet = false;
        }
        this.mExpectedPolicySet.add(e);
        return;
    }
    
    void prune(int n) {
        if (this.isImmutable) {
            throw new IllegalStateException("PolicyNode is immutable");
        }
        if (this.mChildren.size() == 0) {
            return;
        }
        Iterator<PolicyNodeImpl> iterator = this.mChildren.iterator();
        while (iterator.hasNext()) {
            PolicyNodeImpl policyNodeImpl = (PolicyNodeImpl)iterator.next();
            policyNodeImpl.prune(n);
            if (policyNodeImpl.mChildren.size() == 0 && n > this.mDepth + 1) {
                iterator.remove();
            }
            else {
                continue;
            }
        }
        return;
    }
    
    void deleteChild(PolicyNode o) {
        if (this.isImmutable) {
            throw new IllegalStateException("PolicyNode is immutable");
        }
        this.mChildren.remove(o);
        return;
    }
    
    PolicyNodeImpl copyTree() {
        return this.copyTree(null);
    }
    
    private PolicyNodeImpl copyTree(PolicyNodeImpl policyNodeImpl) {
        PolicyNodeImpl policyNodeImpl2 = new PolicyNodeImpl(policyNodeImpl, this);
        Iterator<PolicyNodeImpl> iterator = this.mChildren.iterator();
        while (iterator.hasNext()) {
            ((PolicyNodeImpl)iterator.next()).copyTree(policyNodeImpl2);
        }
        return policyNodeImpl2;
    }
    
    Set<PolicyNodeImpl> getPolicyNodes(int n) {
        HashSet<PolicyNodeImpl> set = new HashSet();
        this.getPolicyNodes(n, set);
        return set;
    }
    
    private void getPolicyNodes(int n, Set<PolicyNodeImpl> set) {
        if (this.mDepth == n) {
            set.add(this);
        }
        else {
            Iterator<PolicyNodeImpl> iterator = this.mChildren.iterator();
            while (iterator.hasNext()) {
                ((PolicyNodeImpl)iterator.next()).getPolicyNodes(n, set);
            }
        }
    }
    
    Set<PolicyNodeImpl> getPolicyNodesExpected(int n, String s, boolean b) {
        if (s.equals("2.5.29.32.0")) {
            return this.getPolicyNodes(n);
        }
        return this.getPolicyNodesExpectedHelper(n, s, b);
    }
    
    private Set<PolicyNodeImpl> getPolicyNodesExpectedHelper(int n, String o, boolean b) {
        HashSet<PolicyNodeImpl> set = new HashSet();
        if (this.mDepth < n) {
            Iterator<PolicyNodeImpl> iterator = this.mChildren.iterator();
            while (iterator.hasNext()) {
                set.addAll(((PolicyNodeImpl)iterator.next()).getPolicyNodesExpectedHelper(n, o, b));
            }
        }
        else if (b) {
            if (this.mExpectedPolicySet.contains("2.5.29.32.0")) {
                set.add(this);
            }
        }
        else if (this.mExpectedPolicySet.contains(o)) {
            set.add(this);
        }
        return set;
    }
    
    Set<PolicyNodeImpl> getPolicyNodesValid(int n, String anObject) {
        HashSet<PolicyNodeImpl> set = new HashSet();
        if (this.mDepth < n) {
            Iterator<PolicyNodeImpl> iterator = this.mChildren.iterator();
            while (iterator.hasNext()) {
                set.addAll(((PolicyNodeImpl)iterator.next()).getPolicyNodesValid(n, anObject));
            }
        }
        else if (this.mValidPolicy.equals(anObject)) {
            set.add(this);
        }
        return set;
    }
    
    private static String policyToString(String s) {
        if (s.equals("2.5.29.32.0")) {
            return "anyPolicy";
        }
        return s;
    }
    
    String asString() {
        if (this.mParent == null) {
            return "anyPolicy  ROOT\n";
        }
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < this.getDepth(); ++i) {
            sb.append("  ");
        }
        sb.append(policyToString(this.getValidPolicy()));
        sb.append("  CRIT: ");
        sb.append(this.isCritical());
        sb.append("  EP: ");
        Iterator<String> iterator = this.getExpectedPolicies().iterator();
        while (iterator.hasNext()) {
            sb.append(policyToString((String)iterator.next()));
            sb.append(" ");
        }
        sb.append(" (");
        sb.append(this.getDepth());
        sb.append(")\n");
        return sb.toString();
    }
}
